import tkinter, tkinter.filedialog
from math import *
import re
#import canvas2svg
import grafico


class Applicazione(tkinter.Frame):
	"""
	Classe dell'applicazione
	Inizializzarla con un finestra creata con Tk().
	"""
	def __init__(self, finestra):
		tkinter.Frame.__init__(self, finestra)

		#Impostazioni della finestra

		finestra.title("Impostazioni")
		finestra.resizable(0, 0)
		finestra.geometry("{}x{}+{}+{}".format(
			645, 110,
			int((finestra.winfo_screenwidth() / 2) - 320),
			finestra.winfo_screenheight()
		))

		#Variabili

		self.tipo = tkinter.StringVar()
		self.testo = tkinter.StringVar()
		self.tipo.set("normale")
		self.grafico = grafico.Grafico(20, 20)
		self.frame = self.grafico.frame
		self.widgets()
		self.scritta()

	def widgets(self):
		"""
		Creare e disegna i widget della finestra.
		"""

		#Etichette

		self.label1 = tkinter.Label(
			finestra,
			textvariable=self.testo
		)
		self.label2 = tkinter.Label(
			finestra,
			text="Intervallo:"
		)
		self.label3 = tkinter.Label(
			finestra,
			text="Colore:"
		)

		#Caselle di testo

		self.casella1 = tkinter.Entry(finestra)
		self.casella2 = tkinter.Entry(finestra)
		self.casella3 = tkinter.Entry(finestra)

		#Opzioni

		self.scelta1 = tkinter.Radiobutton(
			finestra,
			value="normale",
			text="Normale",
			variable=self.tipo,
			command=self.scritta
		)
		self.scelta2 = tkinter.Radiobutton(
			finestra,
			value="punti",
			text="Definita a punti",
			variable=self.tipo,
			command=self.scritta
		)
		self.scelta3 = tkinter.Radiobutton(
			finestra,
			value="tratti",
			text="Definita a tratti",
			variable=self.tipo,
			command=self.scritta
		)

		#Pulsanti

		self.pulsante1 = tkinter.Button(
			finestra,
			text="Disegna",
			command=self.disegna
		)
		self.pulsante2 = tkinter.Button(
			finestra,
			text="Cancella",
			command=self.vuota
		)
		self.pulsante3 = tkinter.Button(
			finestra,
			text="Pulisci Canvas",
			command=self.grafico.pulisci
		)
		self.pulsante4 = tkinter.Button(
			finestra,
			text="Salva",
			command=self.salva
		)
		self.pulsante5 = tkinter.Button(
			finestra,
			text="Esci",
			command=finestra.quit
		)

		#Layout dei widgets

		self.label1.grid(
			column=0, row=0,
			sticky="nw",
			pady=5, padx=5
		)
		self.label2.grid(
			column=0, row=1,
			sticky="nw",
			pady=2.5, padx=5
		)
		self.label3.grid(
			column=0, row=2,
			sticky="nw",
			pady=2.5, padx=5
		)
		self.casella1.grid(
			column=1, row=0,
			sticky="nw",
			pady=5, padx=5
		)
		self.casella2.grid(
			column=1, row=1,
			sticky="nw",
			pady=2.5, padx=5
		)
		self.casella3.grid(
			column=1, row=2,
			padx=5, pady=2.5
		)
		self.scelta1.grid(
			column=2, row=1,
			sticky="nw",
			pady=2.5, padx=5
		)
		self.scelta2.grid(
			column=3, row=1,
			sticky="nw",
			pady=2.5, padx=5
		)
		self.scelta3.grid(
			column=4, row=1,
			sticky="nw",
			pady=2.5, padx=5
		)
		self.pulsante1.grid(
			column=2, row=0,
			sticky="nwes",
			pady=2.5, padx=5
		)
		self.pulsante2.grid(
			column=3, row=0,
			sticky="nwes",
			pady=2.5, padx=5
		)
		self.pulsante3.grid(
			column=4, row=0,
			sticky="nwes",
			pady=2.5, padx=5
			)
		self.pulsante4.grid(
			column=2, row=2,
			sticky="nwes",
			pady=2.5, padx=5,
			columnspan=2
		)
		self.pulsante5.grid(
			column=4, row=2,
			sticky="nwes",
			pady=2.5, padx=5
		)

	def scritta(self):
		"""
		Cambia la scritta in base al tipo di grafico.
		"""
		if self.tipo.get() == "normale":
			self.testo.set("f(x): y =")
		elif self.tipo.get() == "tratti":
			self.testo.set("[f1(x),(intervallo)],[f2(x),(intervallo)],...")
		else:
			self.testo.set("(x1,y1),(x2,y2),...")

	def disegna(self):
		"""
		Disegna il grafico della funzione o dei dati inseriti
		"""

		#Converte la funzione in modo che possa essere interpretata da eval().
		funzione = re.sub(
			"([\+-\-]?\d+)(x)",
			"\\1*x",
			self.casella1.get().replace("^", "**")
		)
		funzione = re.sub(
			"(\|)(.+)(\|)",
			"abs(\\2)",
			funzione
		)
		funzione = re.sub(
			"(.+)(\!)",
			"factorial(\\1)",
			funzione
		)

		#Legge l'intervallo inserito.
		try:
			intervallo = eval(self.casella2.get())
		except SyntaxError:
			inizio = -self.grafico.X
			fine = self.grafico.X
		else:
			inizio = float(intervallo[0])
			fine = float(intervallo[1])

		#Legge il colore inserito.
		colore = self.casella3.get()
		if colore == "":
			colore = "orange"

		#Disegna il grafico in base al tipo inserito.
		if self.tipo.get() == "normale":
			funzione = eval("lambda x:" + funzione)
			self.grafico.disegna(funzione, inizio, fine, colore)
		elif self.tipo.get() == "tratti":
			funzioni = re.sub("(\[\()(.+?\))", "\\1lambda x:\\2", funzione)
			funzioni = re.sub("(\[)", "(", funzioni.replace("]", ")"))
			for funzione in eval(funzioni):
				self.grafico.disegna(
					funzione[0],
					funzione[1][0],
					funzione[1][1],
					colore
				)
		else:
			funzione = grafico.punti(eval(self.casella1.get()))
			self.grafico.disegna(
				funzione,
				punti[0][0],
				punti[len(punti) - 1][1],
				colore
			)

	def vuota(self):
		"""
		Rimuove tutti i dati inseriti.
		"""
		self.casella1.delete(0, "end")
		self.casella2.delete(0, "end")
		self.casella3.delete(0, "end")
		self.finestra.withdraw()

	def salva(self):
		"""
		Salva il grafico in formato svg.
		"""
		opzioni = {
			"parent": self.frame,
			"defaultextension": ".svg",
			"initialfile": "grafico.svg",
			"title": "Salva il grafico"
		}
		file = tkinter.filedialog.asksaveasfilename(**opzioni)
		canvas2svg.saveall(file, self.grafico.canvas)

finestra = tkinter.Tk()
app = Applicazione(finestra)
app.mainloop()
